@@ -20,10 +20,16 @@ rm -rf vendor/pkg
 . ./set_gopath.sh
 mkdir -p bin
 
-for i in bsondump mongostat mongofiles mongoexport mongoimport mongorestore mongodump mongotop mongooplog mongoreplay; do
+ec=0
+for i in bsondump mongostat mongofiles mongoexport mongoimport mongorestore mongodump mongotop mongoreplay; do
         echo "Building ${i}..."
-        go build -o "bin/$i" -tags "$tags" "$i/main/$i.go"
-        ./bin/$i --version
+        go build -o "bin/$i" -tags "$tags" "$i/main/$i.go" || { echo "Error building $i"; ec=1; break; }
+        ./bin/$i --version | head -1
 done
 
+if [ -t /dev/stdin ]; then
+    stty sane
+fi
+
 mv -f common/options/options.go.bak common/options/options.go
+exit $ec
